@@ -1,11 +1,38 @@
-__version__ = '1.0.2'
+__version__ = '1.1.0'
 
 import sys
 
-if float(sys.version[:3]) >= 3.8:
-    print("Please use a Python version with version number SMALLER than 3.8")
-    print("Python versions equal or higher 3.8 is currently NOT SUPPORTED by NLU")
-    exit()
+
+def check_pyspark_install():
+    try :
+        from pyspark.sql import SparkSession
+        try :
+            import sparknlp
+            v = sparknlp.start().version
+            spark_major = int(v.split('.')[0])
+            if spark_major >= 3 :
+                raise Exception()
+        except :
+            print(f"Detected pyspark version={v} Which is >=3.X\nPlease run '!pip install pyspark==2.4.7' or install any pyspark>=2.4.0 and pyspark<3")
+            # print(f"Or set nlu.load(version_checks=False). We disadvise from doing so, until Pyspark >=3 is officially supported in 2021.")
+            return False
+    except :
+        print("No Pyspark installed!\nPlease run '!pip install pyspark==2.4.7' or install any pyspark>=2.4.0 with pyspark<3")
+        return False
+    return True
+
+def check_python_version():
+    if float(sys.version[:3]) >= 3.8:
+        print("Please use a Python version with version number SMALLER than 3.8")
+        print("Python versions equal or higher 3.8 are currently NOT SUPPORTED by NLU")
+        return False
+    return True
+
+
+# if not check_pyspark_install(): raise Exception()
+if not check_python_version(): raise Exception()
+
+
 
 import nlu
 import logging
@@ -53,7 +80,7 @@ from nlu.components.utils.ner_to_chunk_converter import ner_to_chunk_converter
 
 # sentence
 from nlu.components.sentence_detectors.pragmatic_sentence_detector.sentence_detector import PragmaticSentenceDetector
-from nlu.components.sentence_detectors.deep_sentence_detector.deep_sentence_detector import SentenDetectorDeep
+from nlu.components.sentence_detectors.deep_sentence_detector.deep_sentence_detector import SentenceDetectorDeep
 # Embeddings
 from nlu.components.embeddings.albert.spark_nlp_albert import SparkNLPAlbert
 from nlu.components.embeddings.sentence_bert.BertSentenceEmbedding import BertSentence
@@ -90,6 +117,8 @@ from nlu.components.normalizer import Normalizer
 from nlu.components.stopwordscleaner import StopWordsCleaner
 from nlu.components.stemmers.stemmer.spark_nlp_stemmer import SparkNLPStemmer
 from nlu.components.normalizers.normalizer.spark_nlp_normalizer import SparkNLPNormalizer
+from nlu.components.normalizers.document_normalizer.spark_nlp_normalizer import SparkNLPDocumentNormalizer
+
 from nlu.components.lemmatizers.lemmatizer.spark_nlp_lemmatizer import SparkNLPLemmatizer
 from nlu.components.stopwordscleaners.stopwordcleaner.nlustopwordcleaner import NLUStopWordcleaner
 from nlu.components.stopwordscleaner import StopWordsCleaner
@@ -98,6 +127,8 @@ from nlu.components.spell_checkers.norvig_spell.norvig_spell_checker import Norv
 from nlu.components.spell_checkers.context_spell.context_spell_checker import ContextSpellChecker
 from nlu.components.spell_checkers.symmetric_spell.symmetric_spell_checker import SymmetricSpellChecker
 from nlu.components.tokenizers.default_tokenizer.default_tokenizer import DefaultTokenizer
+from nlu.components.tokenizers.word_segmenter.word_segmenter import WordSegmenter
+
 from nlu.components.chunkers.default_chunker.default_chunker import DefaultChunker
 from nlu.components.embeddings_chunks.chunk_embedder.chunk_embedder import ChunkEmbedder
 from nlu.components.chunkers.ngram.ngram import NGram
@@ -107,6 +138,10 @@ from nlu.components.utils.sentence_detector.sentence_detector import SparkNLPSen
 from nlu.components.sentence_detector import NLUSentenceDetector
 from nlu.info import AllComponentsInfo
 
+#seq2seq
+from nlu.components.seq2seqs.marian.marian import Marian
+from nlu.components.seq2seqs.t5.t5 import T5
+from nlu.components.sequence2sequence import Seq2Seq
 global spark_started, spark, active_pipes, all_components_info, nlu_package_location
 
 nlu_package_location = nlu.__file__[:-11]
@@ -115,29 +150,103 @@ active_pipes = []
 spark_started = False
 spark = None
 
+import os
 
-def load(request, verbose=False):
+def read_nlu_info(path):
+    f = open(os.path.join(path,'nlu_info.txt'), "r")
+    nlu_ref = f.readline()
+    f.close()
+    return nlu_ref
+
+
+def is_running_in_databricks():
+    #Check if the currently running Python Process is running in Databricks or not
+    # If any Enviroment Variable name contains 'DATABRICKS' this will return True, otherwise False
+    for k in os.environ.keys() :
+        if 'DATABRICKS' in k :
+            return True
+    return False
+
+def load_nlu_pipe_from_hdd(pipe_path):
+    pipe = NLUPipeline()
+    if nlu.is_running_in_databricks() :
+        if pipe_path.startswith('/dbfs/') or pipe_path.startswith('dbfs/'):
+            nlu_path = pipe_path
+            if pipe_path.startswith('/dbfs/'):
+                nlp_path =  pipe_path.replace('/dbfs','')
+            else :
+                nlp_path =  pipe_path.replace('dbfs','')
+
+        else :
+            nlu_path = 'dbfs/' + pipe_path
+            if pipe_path.startswith('/') : nlp_path = pipe_path
+            else : nlp_path = '/' + pipe_path
+
+        nlu_ref = read_nlu_info(nlu_path)
+        if os.path.exists(pipe_path):
+            # if os.path.exists(info_path):
+            pipe_components = construct_component_from_pipe_identifier(nlu_ref,nlu_ref,'hdd',path=nlp_path)
+            # pipe_components = construct_component_from_pipe_identifier('nlu_ref','nlu_ref','hdd',path=pipe_path)
+
+            for c in pipe_components: pipe.add(c, nlu_ref, pretrained_pipe_component=True)
+
+            return pipe
+            # else:
+            #     print(f'Could not find nlu_info.json file in  {pipe_path}')
+            #     return NluError
+        else :
+            print(f'Could not find nlu pipe folder in {pipe_path}')
+            return NluError
+
+
+    nlu_ref = read_nlu_info(pipe_path)
+    if os.path.exists(pipe_path):
+        # if os.path.exists(info_path):
+        pipe_components = construct_component_from_pipe_identifier(nlu_ref,'nlu_ref','hdd',path=pipe_path)
+        for c in pipe_components: pipe.add(c, nlu_ref, pretrained_pipe_component=True)
+
+        return pipe
+        # else:
+        #     print(f'Could not find nlu_info.json file in  {pipe_path}')
+        #     return NluError
+    else :
+        print(f'Could not find nlu pipe folder in {pipe_path}')
+        return NluError
+def enable_verbose():
+    logger.setLevel(logging.INFO)
+    ch = logging.StreamHandler()
+    ch.setLevel(logging.INFO)
+    logger.addHandler(ch)
+
+
+def load(request ='from_disk', path=None,verbose=False,version_checks=True):
     '''
     Load either a prebuild pipeline or a set of components identified by a whitespace seperated list of components
     :param verbose:
+    :param path: If path is not None, the model/pipe for the NLU reference will be loaded from the path. Useful for offline mode. Currently only loading entire NLU pipelines is supported, but not loading singular pipes
     :param request: A NLU model/pipeline/component reference
-    :return: returns a fitted nlu pipeline object
+    :param version_checks: Wether to check if Pyspark is properly installed and if the Pyspark version is correct for the NLU version. If set to False, these tests will be skipped
+    :return: returns a non fitted nlu pipeline object
     '''
     gc.collect()
-
+    # if version_checks : check_pyspark_install()
     spark = sparknlp.start()
     spark.catalog.clearCache()
-    # spark.conf.set("spark.sql.execution.arrow.enabled", "true")
     spark_started = True
     if verbose:
-        logger.setLevel(logging.INFO)
-        ch = logging.StreamHandler()
-        ch.setLevel(logging.INFO)
-        logger.addHandler(ch)
+        enable_verbose()
 
     try:
+
+        if path != None :
+            logger.info(f'Trying to load nlu pipeline from local hard drive, located at {path}')
+            pipe = load_nlu_pipe_from_hdd(path)
+            return pipe
         components_requested = request.split(' ')
         pipe = NLUPipeline()
+        language = parse_language_from_nlu_ref(request)
+        pipe.lang=language
+        pipe.nlu_ref = request
         for nlu_ref in components_requested:
             nlu_ref.replace(' ', '')
             # component = component.lower()
@@ -165,7 +274,17 @@ def load(request, verbose=False):
     return pipe
 
 
-def get_default_component_of_type(missing_component_type):
+def parse_language_from_nlu_ref(nlu_ref):
+
+    infos = nlu_ref.split('.')
+    for split in infos:
+        if split in nlu.AllComponentsInfo().all_languages:
+            logger.info(f'Parsed Nlu_ref={nlu_ref} as lang={split}')
+            return split
+    logger.info(f'Parsed Nlu_ref={nlu_ref} as lang=en')
+    return 'en'
+
+def get_default_component_of_type(missing_component_type,language='en'):
     '''
     This function returns a default component for a missing component type.
     It is used to auto complete pipelines, which are missng required components.
@@ -180,7 +299,7 @@ def get_default_component_of_type(missing_component_type):
         if missing_component_type == 'document': return Util('document_assembler')
         if missing_component_type == 'sentence': return Util('sentence_detector')
         if missing_component_type == 'sentence_embeddings': return Embeddings('use')
-        if 'token' in missing_component_type: return nlu.components.tokenizer.Tokenizer("default_tokenizer")
+        if 'token' in missing_component_type: return nlu.components.tokenizer.Tokenizer("default_tokenizer", language=language)
         if missing_component_type == 'word_embeddings': return Embeddings(nlu_ref='glove')
         if missing_component_type == 'pos':   return Classifier(nlu_ref='pos')
         if missing_component_type == 'ner':   return Classifier(nlu_ref='ner')
@@ -197,10 +316,10 @@ def get_default_component_of_type(missing_component_type):
         # if there is an @ in the name, we must get some specific pretrained model from the sparknlp reference that should follow after the @
         missing_component_type, sparknlp_reference = missing_component_type.split('@')
         if 'embed' in missing_component_type:
-            return construct_component_from_identifier(language='en', component_type='embed',
+            return construct_component_from_identifier(language=language, component_type='embed',
                                                        nlp_ref=sparknlp_reference)
         if 'pos' in missing_component_type or 'ner' in missing_component_type:
-            return construct_component_from_identifier(language='en', component_type='classifier',
+            return construct_component_from_identifier(language=language, component_type='classifier',
                                                        nlp_ref=sparknlp_reference)
         if 'chunk_embeddings' in missing_component_type:
             return embeddings_chunker.EmbeddingsChunker()
@@ -214,7 +333,7 @@ def get_default_component_of_type(missing_component_type):
         logger.exception("Could not resolve default component type for missing type=%s", missing_component_type)
 
 
-def parse_component_data_from_name_query(nlu_reference, detect_lang=False):
+def parse_component_data_from_name_query(nlu_reference, detect_lang=False, path=None):
     '''
     This method implements the main namespace for all component names. It parses the input request and passes the data to a resolver method which searches the namespace for a Component for the input request
     It returns a list of NLU.component objects or just one NLU.component object alone if just one component was specified.
@@ -225,12 +344,17 @@ def parse_component_data_from_name_query(nlu_reference, detect_lang=False):
     This method will parse <language>.<NLU_action>
         Additional data about dataset and variant will be resolved by corrosponding action classes
 
+    If train prefix is part of the nlu_reference ,the trainable namespace will e searched
+
+    if 'translate_to' or 'marian' is inside of the nlu_reference, 'xx' will be prefixed to the ref and set as lang if it is not already
+    Since all translate models are xx lang
     :param nlu_reference: User request (should be a NLU reference)
     :param detect_lang: Wether to automatically  detect language
-    :return: Pipeline or component for the NLU reference
+    :return: Pipeline or component for the NLU reference.
     '''
 
     infos = nlu_reference.split('.')
+
     if len(infos) < 0:
         logger.exception("EXCEPTION: Could not create a component for nlu reference=%s", nlu_reference)
         return NluError()
@@ -238,14 +362,26 @@ def parse_component_data_from_name_query(nlu_reference, detect_lang=False):
     component_type = ''
     dataset = ''
     component_embeddings = ''
-    component_pipe = []
+    trainable = False
+    resolved_component = None
     # 1. Check if either a default cmponent or one specific pretrained component or pipe  or alias of them is is requested without more sepcificatin about lang,dataset or embeding.
     # I.e. 'explain_ml' , 'explain; 'albert_xlarge_uncased' or ' tokenize'  or 'sentiment' s requested. in this case, every possible annotator must be checked.
     # format <class> or <nlu identifier>
     # type parsing via checking parts in nlu action <>
+
+
     if len(infos) == 0:
         logger.exception("Split  on query is 0.")
     # Query of format <class>, no embeds,lang or dataset specified
+    elif 'train' in infos :
+        trainable = True
+        component_type = infos[1]
+    elif 'translate_to' in nlu_reference and not 't5' in nlu_reference:
+        language = 'xx'
+        if nlu_reference[0:3] != 'xx.':
+            nlu_reference = 'xx.' + nlu_reference
+        logger.info(f'Setting lang as xx for nlu_ref={nlu_reference}')
+        component_type = 'translate_to'
     elif len(infos) == 1:
         # if we only have 1 split result, it must a a NLU action reference or an alias
         logger.info('Setting default lang to english')
@@ -256,7 +392,7 @@ def parse_component_data_from_name_query(nlu_reference, detect_lang=False):
     elif infos[0] in all_components_info.all_languages:
         language = infos[0]
         component_type = infos[1]
-
+        logger.info(f"Got request for trainable model {component_type}")
         if len(infos) == 3:  # dataset specified
             dataset = infos[2]
         if len(infos) == 4:  # embeddings specified
@@ -284,16 +420,14 @@ def parse_component_data_from_name_query(nlu_reference, detect_lang=False):
     logger.info(
         'For input nlu_ref %s detected : \n lang: %s  , component type: %s , component dataset: %s , component embeddings  %s  ',
         nlu_reference, language, component_type, dataset, component_embeddings)
-    resolved_component = resolve_component_from_parsed_query_data(language, component_type, dataset,
-                                                                  component_embeddings, nlu_reference)
-
+    resolved_component = resolve_component_from_parsed_query_data(language, component_type, dataset,component_embeddings, nlu_reference, trainable)
     if resolved_component is None:
         logger.exception("EXCEPTION: Could not create a component for nlu reference=%s", nlu_reference)
         return NluError()
     return resolved_component
 
 
-def resolve_component_from_parsed_query_data(language, component_type, dataset, component_embeddings, nlu_ref):
+def resolve_component_from_parsed_query_data(language, component_type, dataset, component_embeddings, nlu_ref,trainable=False,path=None):
     '''
     Searches the NLU name spaces for a matching NLU reference. From that NLU reference, a SparkNLP reference will be aquired which resolved to a SparkNLP pretrained model or pipeline
     :param nlu_ref: Full request which was passed to nlu.load()
@@ -308,12 +442,20 @@ def resolve_component_from_parsed_query_data(language, component_type, dataset,
     logger.info('Searching local Namespaces for SparkNLP reference.. ')
     resolved = False
 
+    # 0. check trainable references
+    if trainable == True :
+        if nlu_ref in NameSpace.trainable_models.keys():
+            component_kind = 'trainable_model'
+            nlp_ref = NameSpace.trainable_models[nlu_ref]
+            logger.info(f'Found Spark NLP reference in trainable models namespace = {nlp_ref}')
+            resolved = True
+
     # 1. check if pipeline references for resolution
     if resolved == False and language in NameSpace.pretrained_pipe_references.keys():
         if nlu_ref in NameSpace.pretrained_pipe_references[language].keys():
             component_kind = 'pipe'
             nlp_ref = NameSpace.pretrained_pipe_references[language][nlu_ref]
-            logger.info('Found Spark NLP reference in pretrained pipelines namespace')
+            logger.info(f'Found Spark NLP reference in pretrained pipelines namespace = {nlp_ref}')
             resolved = True
 
     # 2. check if model references for resolution
@@ -321,7 +463,7 @@ def resolve_component_from_parsed_query_data(language, component_type, dataset,
         if nlu_ref in NameSpace.pretrained_models_references[language].keys():
             component_kind = 'model'
             nlp_ref = NameSpace.pretrained_models_references[language][nlu_ref]
-            logger.info('Found Spark NLP reference in pretrained models namespace')
+            logger.info(f'Found Spark NLP reference in pretrained models namespace = {nlp_ref}')
             resolved = True
 
     # 2. check if alias/default references for resolution
@@ -332,6 +474,12 @@ def resolve_component_from_parsed_query_data(language, component_type, dataset,
         logger.info('Found Spark NLP reference in language free aliases namespace')
         resolved = True
 
+        if len(sparknlp_data) > 2 :
+            dataset=sparknlp_data[2]
+        if len(sparknlp_data) > 3 :
+            # special case overwrite for T5
+            nlu_ref=sparknlp_data[3]
+
     # 3. If reference is none of the Namespaces, it must be a component like tokenizer or YAKE or Chunker etc....
     # If it is not, then it does not exist and will be caught later
     if not resolved:
@@ -349,7 +497,7 @@ def resolve_component_from_parsed_query_data(language, component_type, dataset,
             return NluError
         else:
             return constructed_components
-    elif component_kind == 'model' or 'component':
+    elif component_kind in ['model', 'component']:
         constructed_component = construct_component_from_identifier(language, component_type, dataset,
                                                                     component_embeddings, nlu_ref,
                                                                     nlp_ref)
@@ -361,6 +509,14 @@ def resolve_component_from_parsed_query_data(language, component_type, dataset,
             return NluError
         else:
             return constructed_component
+    elif component_kind == 'trainable_model':
+        constructed_component = construct_trainable_component_from_identifier(nlu_ref,nlp_ref)
+        if constructed_component is None:
+            logger.exception(f'EXCEPTION : Could not create NLU component for nlp_ref={nlp_ref} and nlu_ref={nlu_ref}')
+            return NluError
+        else:
+            constructed_component.component_info.is_untrained = True
+            return constructed_component
     else:
         logger.exception(
             "EXCEPTION : Could not resolve query=%s for kind=%s and reference=%s in any of NLU's namespaces ", nlu_ref,
@@ -368,8 +524,51 @@ def resolve_component_from_parsed_query_data(language, component_type, dataset,
             nlp_ref)
         return NluError
 
+import sparknlp
+sparknlp.start()
+def construct_trainable_component_from_identifier(nlu_ref,nlp_ref):
+    '''
+    This method returns a Spark NLP annotator Approach class embelished by a NLU component
+    :param nlu_ref: nlu ref to the trainable model
+    :param nlp_ref: nlp ref to the trainable model
+    :return: trainable model as a NLU component
+    '''
 
-def construct_component_from_pipe_identifier(language, nlp_ref, nlu_ref):
+    logger.info(f'Creating trainable NLU component for nlu_ref = {nlu_ref} and nlp_ref = {nlp_ref}')
+    try:
+        if nlu_ref in ['train.deep_sentence_detector','train.sentence_detector']:
+            #no label col but trainable?
+            return  nlu.NLUSentenceDetector(annotator_class = 'deep_sentence_detector', trainable='True')
+        if nlu_ref in ['train.context_spell','train.spell'] :
+            pass
+        if nlu_ref in ['train.symmetric_spell'] :
+            pass
+        if nlu_ref in ['train.norvig_spell'] :
+            pass
+        if nlu_ref in ['train.unlabeled_dependency_parser'] :
+            pass
+        if nlu_ref in ['train.labeled_dependency_parser'] :
+            pass
+        if nlu_ref in ['train.classifier_dl','train.classifier'] :
+            return nlu.Classifier(annotator_class = 'classifier_dl', trainable=True)
+        if nlu_ref in ['train.ner','train.named_entity_recognizer_dl'] :
+            return nlu.Classifier(annotator_class = 'ner', trainable=True)
+        if nlu_ref in ['train.sentiment_dl','train.sentiment'] :
+            return nlu.Classifier(annotator_class = 'sentiment_dl', trainable=True)
+        if nlu_ref in ['train.vivekn_sentiment'] :
+            pass
+        if nlu_ref in ['train.pos'] :
+            return nlu.Classifier(annotator_class = 'pos', trainable=True)
+        if nlu_ref in ['train.multi_classifier'] :
+            return nlu.Classifier(annotator_class = 'multi_classifier', trainable=True)
+
+
+
+    except:  # if reference is not in namespace and not a component it will cause a unrecoverable crash
+        logger.exception(f'EXCEPTION: Could not create trainable NLU component for nlu_ref = {nlu_ref} and nlp_ref = {nlp_ref}')
+        return None
+
+def construct_component_from_pipe_identifier(language, nlp_ref, nlu_ref,path=None):
     '''
     # creates a list of components from a Spark NLP Pipeline reference
     # 1. download pipeline
@@ -378,14 +577,23 @@ def construct_component_from_pipe_identifier(language, nlp_ref, nlu_ref):
     :param nlu_ref:
     :param language: language of the pipeline
     :param nlp_ref: Reference to a spark nlp petrained pipeline
+    :param path: Load pipe from HDD
     :return: Each element of the SaprkNLP pipeline wrapped as a NLU componed inside of a list
     '''
     logger.info("Starting Spark NLP to NLU pipeline conversion process")
-    from sparknlp.pretrained import PretrainedPipeline
+    from sparknlp.pretrained import PretrainedPipeline, LightPipeline
     if 'language' in nlp_ref: language = 'xx'  # special edge case for lang detectors
-    pipe = PretrainedPipeline(nlp_ref, lang=language)
+    if path == None :
+        pipe = PretrainedPipeline(nlp_ref, lang=language)
+        iterable_stages = pipe.light_model.pipeline_model.stages
+    else :
+        pipe = LightPipeline(PipelineModel.load(path=path))
+        iterable_stages = pipe.pipeline_model.stages
     constructed_components = []
-    for component in pipe.light_model.pipeline_model.stages:
+
+    # for component in pipe.light_model.pipeline_model.stages:
+    for component in iterable_stages:
+
         logger.info("Extracting model from Spark NLP pipeline: %s and creating Component", component)
         parsed = str(component).split('_')[0].lower()
         logger.info("Parsed Component for : %s", parsed)
@@ -400,7 +608,7 @@ def construct_component_from_pipe_identifier(language, nlp_ref, nlu_ref):
             constructed_components.append(nlu.Classifier(model=component, nlp_ref='multiclassifierdl'))
         elif isinstance(component, PerceptronModel):
             constructed_components.append(nlu.Classifier(nlp_ref='classifierdl', model=component))
-        elif isinstance(component, ClassifierDl):
+        elif isinstance(component, (ClassifierDl,ClassifierDLModel)):
             constructed_components.append(nlu.Classifier(nlp_ref='classifierdl', model=component))
         elif isinstance(component, UniversalSentenceEncoder):
             constructed_components.append(nlu.Embeddings(model=component, nlp_ref='use'))
@@ -471,7 +679,10 @@ def construct_component_from_pipe_identifier(language, nlp_ref, nlu_ref):
             constructed_components.append(nlu.StopWordsCleaner(model=component))
         elif isinstance(component, (TextMatcherModel, RegexMatcherModel, DateMatcher,MultiDateMatcher)) or parsed == 'match':
             constructed_components.append(nlu.Matcher(model=component))
-
+        elif isinstance(component,(T5Transformer)):
+            constructed_components.append(nlu.Seq2Seq(annotator_class='t5', model=component))
+        elif isinstance(component,(MarianTransformer)):
+            constructed_components.append(nlu.Seq2Seq(annotator_class='marian', model=component))
         else:
             logger.exception(
                 f"EXCEPTION: Could not infer component type for lang={language} and nlp_ref={nlp_ref} and model {component} during pipeline conversion,")
@@ -503,8 +714,12 @@ def construct_component_from_identifier(language, component_type='', dataset='',
                 component_type, nlp_ref, dataset, language, nlu_ref)
     try:
 
+        if any(
+            x in NameSpace.seq2seq for x in [nlp_ref, nlu_ref, dataset, component_type, ]):
+            return Seq2Seq(annotator_class=component_type, language=language, get_default=False, nlp_ref=nlp_ref,configs=dataset)
+
         # if any([component_type in NameSpace.word_embeddings,dataset in NameSpace.word_embeddings, nlu_ref in NameSpace.word_embeddings, nlp_ref in NameSpace.word_embeddings]):
-        if any(x in NameSpace.word_embeddings and not x in NameSpace.classifiers for x in
+        elif any(x in NameSpace.word_embeddings and not x in NameSpace.classifiers for x in
                [nlp_ref, nlu_ref, dataset, component_type, ] + dataset.split('_')):
             return Embeddings(get_default=False, nlp_ref=nlp_ref, nlu_ref=nlu_ref, language=language)
 
@@ -533,17 +748,18 @@ def construct_component_from_identifier(language, component_type='', dataset='',
             return nlu.lemmatizer.Lemmatizer(language=language, nlp_ref=nlp_ref)
 
         elif any('norm' in x for x in [nlp_ref, nlu_ref, dataset, component_type]):
-            return nlu.normalizer.Normalizer()
+            return nlu.normalizer.Normalizer(nlp_ref=nlp_ref, nlu_ref=nlu_ref)
         elif any('clean' in x or 'stopword' in x for x in [nlp_ref, nlu_ref, dataset, component_type]):
             return nlu.StopWordsCleaner(language=language, get_default=False, nlp_ref=nlp_ref)
         elif any('sentence_detector' in x for x in [nlp_ref, nlu_ref, dataset, component_type]):
-            return NLUSentenceDetector(nlu_ref=nlu_ref, nlp_ref=nlp_ref)
+            return NLUSentenceDetector(nlu_ref=nlu_ref, nlp_ref=nlp_ref, language=language)
 
         elif any('match' in x for x in [nlp_ref, nlu_ref, dataset, component_type]):
             return Matcher(nlu_ref=nlu_ref, nlp_ref=nlp_ref)
 
-        elif any('tokenize' in x for x in [nlp_ref, nlu_ref, dataset, component_type]):
-            return nlu.tokenizer.Tokenizer()
+# THIS NEEDS TO CAPTURE THE WORD SEGMNETER!!!
+        elif any('tokenize' in x or 'segment_words' in x for x in [nlp_ref, nlu_ref, dataset, component_type]):
+            return nlu.tokenizer.Tokenizer(nlp_ref=nlp_ref, nlu_ref=nlu_ref, language=language,get_default=False)
 
         elif any('stem' in x for x in [nlp_ref, nlu_ref, dataset, component_type]):
             return Stemmer()
@@ -694,11 +910,24 @@ def print_all_model_kinds_for_action_and_lang(lang, action):
         if ref_action == action:
             print("nlu.load('" + nlu_reference + "') returns Spark NLP model " + nlp_reference)
 
+def print_trainable_components():
+    '''
+    Print every trainable Algorithm/Model
+    :return: None
+    '''
+    i = 1
+    print('The following models can be trained with a dataset that provides a label column and matching dataset')
+    for name, infos in nlu.all_components_info.all_components.items() :
+        if infos.trainable == True :
+            print(f' {i}. {name}')
+            i+=1
+
+
 
 class NluError:
     def __init__(self):
         pass
-
+        self.has_trainable_components = False
     @staticmethod
     def predict(text, output_level='error', positions='error', metadata='error', drop_irrelevant_cols=False):
         print(
