@@ -1,7 +1,6 @@
 import nlu.pipe_components
 import sparknlp
 from sparknlp.annotator import *
-
 class ClassifierDl:
     @staticmethod
     def get_default_model():
@@ -14,17 +13,24 @@ class ClassifierDl:
         return ClassifierDLModel.pretrained(name,language) \
             .setInputCols("sentence_embeddings") \
             .setOutputCol("category")
-    
+
 
 
 
     @staticmethod
-    def get_default_trainable_model():
+    def get_trainable_model():
         return ClassifierDLApproach() \
+            .setInputCols("sentence_embeddings") \
+            .setOutputCol("category") \
+            .setLabelColumn("y") \
+            .setMaxEpochs(3) \
+           .setEnableOutputLogs(True)
+
+    @staticmethod
+    def get_offline_model(path):
+        return ClassifierDLModel.load() \
             .setInputCols("sentence_embeddings") \
             .setOutputCol("category") \
             .setLabelColumn("label") \
-            .setBatchSize(64) \
-            .setMaxEpochs(20) \
-            .setLr(0.5) \
-            .setDropout(0.5)
\ No newline at end of file
+            .setEnableOutputLogs(True)
+
